/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/* Library includes. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"


#include "command.h"
#include "movement.h"



#define UART_WIFLY 0
#define UART_DEBUG 3

/* The tasks to be created.  Two instances are created of the sender task while
only a single instance is created of the receiver task. */
static void vCmdDownloaderTask( void *pvParameters );
static void vCmdExecutorTask( void *pvParameters );


static void vMotorTestTask( void *pvParameters );
static void vDummyTask( void *pvParameters );
/*-----------------------------------------------------------*/

/* Declare a variable of type xQueueHandle.  This is used to store the queue
that is accessed by all three tasks. */
xQueueHandle xQueue;


int main( void )
{
	/* Init the semi-hosting. */
	printf( "\n" );
	/* The queue is created to hold a maximum of 10 char[20] values. */
    xQueue = xQueueCreate( 10, 20*sizeof( char ) );


	if( xQueue != NULL )
	{
		/* Create one instances of the task that will write to the queue. The task is created at
		priority 1. */

		//xTaskCreate( vDummyTask, "Dummy", 240, NULL, 1, NULL );//
		xTaskCreate( vCmdDownloaderTask, "CmdDownloader", 240, NULL, 1, NULL );
		/* Create the task that will read from the queue.  The task is created with
		priority 2, so above the priority of the sender task. */
		//xTaskCreate( vMotorTestTask, "MotorTest", 240, NULL, 2, NULL );
		xTaskCreate( vCmdExecutorTask, "CmdExecutor", 240, NULL, 2, NULL );

		/* Start the scheduler so the created tasks start executing. */
		vTaskStartScheduler();
	}
	else
	{
		/* The queue could not be created. */
	}

    /* If all is well we will never reach here as the scheduler will now be
    running the tasks.  If we do reach here then it is likely that there was
    insufficient heap memory available for a resource to be created. */
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/
static void vMotorTestTask( void *pvParameters ){
	while (1){

		printfString1(UART_DEBUG, "stop");
		movement_stop();
		movement_start();
		printfString1(UART_DEBUG, "start");
		printfString1(UART_DEBUG, "moveForward");
		movement_Forward();
		movement_stop();
		printfString1(UART_DEBUG, "moveBack");
		movement_Back();
		movement_stop();
		printfString1(UART_DEBUG, "turnLeft");
		movement_turnLeft();
		movement_stop();
		printfString1(UART_DEBUG, "turnRight");
		movement_turnRight();
		movement_stop();

	}

}


void vDummyTask( void *pvParameters )
{

	while(1)
	{
		taskYIELD();//return the processor to the scheduler
	}
}
static void vCmdDownloaderTask( void *pvParameters )
{
	char sValueToSend[20];
	sValueToSend[0]='\0';
	int result=0;
	portBASE_TYPE xStatus;

	printfString1(UART_DEBUG, "Download task awaiting new commands");

	/* As per most tasks, this task is implemented within an infinite loop. */
	for( ;; )
	{
		result=getCommand(sValueToSend);

		if (result <0){
			continue;//bad command, try again
		}


		/* The first parameter is the queue to which data is being sent.  The
		queue was created before the scheduler was started, so before this task
		started to execute.

		The second parameter is the address of the data to be sent.

		The third parameter is the Block time � the time the task should be kept
		in the Blocked state to wait for space to become available on the queue
		should the queue already be full.  In this case we don�t specify a block
		time because there should always be space in the queue. */


		xStatus = xQueueSendToBack( xQueue, &sValueToSend, 0 );

		if( xStatus != pdPASS )
		{
			/* We could not write to the queue because it was full  */
			printfString1(UART_DEBUG,  "Could not send to the queue.\r\n" );
		}
		printfString1(UART_DEBUG, "Sent : ");
		printfString1(UART_DEBUG, sValueToSend);
		/* Allow other tasks to execute. */
		taskYIELD();
	}
}
/*-----------------------------------------------------------*/

static void vCmdExecutorTask( void *pvParameters )
{
	/* Declare the variable that will hold the values received from the queue. */
	char sReceivedValue[20];
	sReceivedValue[0]='\0';

	portBASE_TYPE xStatus;
	const portTickType xTicksToWait = 300000 / portTICK_RATE_MS;

	printfString1(UART_DEBUG, "Entering command executor task");
		/* This task is also defined within an infinite loop. */

	for( ;; )
	{

		/* As this task unblocks immediately that data is written to the queue this
		call should always find the queue empty. */
		if( uxQueueMessagesWaiting( xQueue ) != 0 )
		{
			printfString1(UART_DEBUG, "Queue should have been empty!\r\n" );
		}

		/* The first parameter is the queue from which data is to be received.  The
		queue is created before the scheduler is started, and therefore before this
		task runs for the first time.

		The second parameter is the buffer into which the received data will be
		placed.  In this case the buffer is simply the address of a variable that
		has the required size to hold the received data.

		the last parameter is the block time � the maximum amount of time that the
		task should remain in the Blocked state to wait for data to be available should
		the queue already be empty. */
		xStatus = xQueueReceive( xQueue, &sReceivedValue, xTicksToWait );

		if( xStatus == pdPASS )
		{
			/* Data was successfully received from the queue, execute the appropriate command. */
			printfString1(UART_DEBUG, sReceivedValue);
			processCommand(sReceivedValue);

		}
		else
		{
			/* We did not receive anything from the queue .
			 */
			printfString1(UART_DEBUG, "Could not receive from the queue.\r\n");
		}
	}
}
/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	//for( ;; );

}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}



