/*
 * command.c
 *
 *  Created on: 06/12/2012
 *      Author: Cesar
 */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/* Demo includes. */

#include "basic_io.h"
#include "printf_library.h"
#include "wifly.h"
#include "uart.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "command.h"
#include "movement.h"

static xSemaphoreHandle xMutex;

/*
 * int commandInit()
 * Function that initializes the wifly module to allow for incoming tcp connections
 * Input parameters:
 * 		- wiflyConfig: 0 to skip tcp configuration if it has already been done; 1 otherwise
 * Output parameters: none
 * Returns:	-1 in case of error, 0 otherwise
 */
int commandInit(int wiflyConfig){

	/* wifly module initialization */
	if (DEBUG_MODE==1){
	printfString1(UART_DEBUG, "--------------");
	printfString1(UART_DEBUG, "");
	printfString1(UART_DEBUG, "Initialization:  ");
	printfString1(UART_DEBUG, "");
	printfString1(UART_DEBUG, "Wifly reset:  ");
	}

	int result = wifly_reset();
	if (result < 0) {
		if (DEBUG_MODE==1)printfString1(UART_DEBUG, "ERROR");
		return -1;
	}else{
		if (DEBUG_MODE==1)printfString1(UART_DEBUG, "OK");
	}

	if (DEBUG_MODE==1)printfString1(UART_DEBUG, "TCP config:  ");

	if (wiflyConfig==1){
		result = wifly_setTcpServerConfig();
	}else{
		result = 0;
	}


	if (result < 0) {
		if (DEBUG_MODE==1)printfString1(UART_DEBUG, "ERROR");
		return -1;
	}else{
		if (DEBUG_MODE==1)printfString1(UART_DEBUG, "OK");
	}


	if (DEBUG_MODE==1)printfString1(UART_DEBUG, "Wifly connect:  ");
	result = wifly_connect();

	if (result < 0) {
		if (DEBUG_MODE==1)printfString1(UART_DEBUG, "ERROR");
		return -1;
	}else{
		if (DEBUG_MODE==1)printfString1(UART_DEBUG, "OK");
	}
	int order=0;
	const char * ssid = "AndroidHotspot";
	const char * passwd = "";
	int security=0;


	if (DEBUG_MODE==1)printfString1(UART_DEBUG, "Network join:  ");
	result = wifly_join(order,ssid,security,passwd);

	if (result < 0) {
		if (DEBUG_MODE==1)printfString1(UART_DEBUG, "ERROR");
		return -1;
	}else{
		if (DEBUG_MODE==1)printfString1(UART_DEBUG, "OK");
	}



	if (DEBUG_MODE==1){
		printfString1(UART_DEBUG, "");
		printfString1(UART_DEBUG, "Initialization:  OK");
		printfString1(UART_DEBUG, "--------------");
	}
	/* end of wifly initialization */

	/* set up access semaphore */
	xMutex = xSemaphoreCreateMutex();


	return 0;


}



/*
 * int getCommand(char * comanda)
 * Function that retrieves the text received in an incoming tcp connection,
 * extracts the corresponding command from it, and prepares a response to be sent in html format
 * on the same tcp connection. The chunked protocol is used to send the body of the response
 * so that the client browser knows when the transfer is finished. This is needed because a known bug in the
 * firmware of the wifly module, that fails to send a proper tcp session FIN command to the client
 *
 * Input parameters: none
 * Output parameters:
 * 		- comanda: pointer to the received command string
 * Returns:	-1 in case of error, 0 otherwise
 */

int getCommand(char * comanda) {
	if( xMutex== NULL){
		commandInit(1);
	}
	comanda[0]='\0';
	int retorn = -1;

	xSemaphoreTake( xMutex, portMAX_DELAY );
	{

		char * tcpsession=printf_captura_UART0(200);
		char linia[127];
		linia[0]='\0';

		//El text rebut en una sessió tcp en rebre una comanda és com segueix:

		// OPEN*GET /move_forward HTTP/1.1
		// Host: 192.168.43.69
		// User-Agent: Mozilla/5.0 (Windows NT 6.1; WOW64; rv:13.0) Gecko/20100101 Firefox/13.0.1
		// Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8
		// Accept-Language: ca,en-us;q=0.7,en;q=0.3
		// Accept-Encoding: gzip, deflate
		// DNT: 1
		// Connection: keep-alive
		//
		// *CLOS*

		if(strlen(tcpsession)>0){
			if (DEBUG_MODE==1)printfString1(UART_DEBUG, tcpsession);

			  char * pch;

			  pch = strtok (tcpsession,"/ ");
			  while (pch !=NULL){
				  if (strstr(pch, "cmd_") != NULL ){//extracts the command, which is between a / bar and a space
					  pch = strtok (pch, "_");
					  pch = strtok (NULL, "_");
					  break;
				  }
				  pch = strtok (NULL, "/ ");
			  }

			  if (pch !=NULL){
					comanda[0]='\0';
					strcpy(comanda,pch);
					retorn=strlen(pch);
					free(pch);
					free(tcpsession);

					printfString1(UART_WIFLY, "");
					printfString1(UART_WIFLY, "HTTP/1.1 200 OK");
					printfString1(UART_WIFLY, "Content-Type: text/html");
					printfString1(UART_WIFLY, "Transfer-Encoding: chunked");
					printfString1(UART_WIFLY, "");

					printfString1(UART_WIFLY, chunk_line("<html><title>Rhino1 Cmd Sender</title><body><h1>"));
					printfString1(UART_WIFLY, chunk_line("Command <b>"));
					printfString1(UART_WIFLY, chunk_line(comanda));
					printfString1(UART_WIFLY, chunk_line("</b> received</h1>"));
					printfString1(UART_WIFLY, chunk_line("</body></html>"));
					printfString1(UART_WIFLY, chunk_line(""));
					wifly_closeTcpConnection();
			  }else{
					free(tcpsession);

					printfString1(UART_WIFLY, "");
					printfString1(UART_WIFLY, "HTTP/1.1 200 OK");
					printfString1(UART_WIFLY, "Content-Type: text/html");
					printfString1(UART_WIFLY, "Transfer-Encoding: chunked");
					printfString1(UART_WIFLY, "");
					printfString1(UART_WIFLY, chunk_line("<html><title>Rhino1 Cmd Sender</title><body><h1>"));
					printfString1(UART_WIFLY, chunk_line("Command not understood"));
					printfString1(UART_WIFLY, chunk_line("</h1></body></html>"));
					printfString1(UART_WIFLY, chunk_line(""));
					//printfString1(UART_WIFLY, "");
					wifly_closeTcpConnection();

					retorn=-1;
			  }
		}else{
			retorn=-1;
		}
	}
	xSemaphoreGive( xMutex );
	return retorn;
}

/*
 * int processCommand(char * comanda)
 * Function that given a command string executes the appropriate function
 * of the motor control library movementlib.
 *  * Input parameters:
 * 		- comanda: pointer to the command string to execute
 * Output parameters: none
 * Returns:	-1 in case of error, 0 otherwise
 */
int processCommand(char * comanda){
	if( xMutex== NULL){
		commandInit(1);
	}


	int resultat=0;
	if (strstr(comanda, "test") != NULL ){
		printfString1(UART_DEBUG, "dummy command received" );
	}else if (strstr(comanda, "moveForward") != NULL ){
		resultat=movement_Forward();
	}else if (strstr(comanda, "moveBack") != NULL ){
		resultat=movement_Back();
	}else if (strstr(comanda, "turnLeft") != NULL ){
		resultat=movement_turnLeft();
	}else if (strstr(comanda, "turnRight") != NULL ){
		resultat=movement_turnRight();
	}else if (strstr(comanda, "start") != NULL ){
		resultat=movement_start();
	}else if (strstr(comanda, "stop") != NULL ){
		resultat=movement_stop();
	}else if (strstr(comanda, "pause") != NULL ){
		resultat=movement_pause();
	}else if (strstr(comanda, "max2D") != NULL ){
		//only commands that use the wifly module need the semaphore protection
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			resultat=movement_ToMaxRSSI_2D();
		}
		xSemaphoreGive( xMutex );

	}else if (strstr(comanda, "maxRSSIstep") != NULL ){
		//only commands that use the wifly module need the semaphore protection
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			resultat=movement_ToMaxRSSI_1D(1);
		}
		xSemaphoreGive( xMutex );

	}else if (strstr(comanda, "maxRSSI") != NULL ){
		//only commands that use the wifly module need the semaphore protection
		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
			resultat=movement_ToMaxRSSI_1D(0);
		}
		xSemaphoreGive( xMutex );

	}else{
		printfString1(UART_DEBUG, "bad command" );
	}
	return 0;
}
