/*
 * printf_library.c
 *
 *  Created on: 04/11/2012
 *      Author: Cesar
 */

#include <stdio.h>
//#include "consoleprint.h"

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "uart.c"
#include "printf_library.h"

#include <string.h>
#include <stdlib.h>
static char cBuffer[BUFSIZE];

extern volatile uint32_t UART3Count;
extern volatile uint8_t UART3Buffer[BUFSIZE];
extern volatile uint32_t UART0Count;
extern volatile uint8_t UART0Buffer[BUFSIZE];
extern volatile uint32_t UART1Count;
extern volatile uint8_t UART1Buffer[BUFSIZE];


/*
 * void printfStringInit( int debug)
 * Procedure that sends a \r\n terminated string message to the uart interface specified by uart_number, and optionally retrieves the response
 * Input parameters:
 * 		- debug: 1 if the debug UART channel must be enabled, 0 otherwise
 * Output parameters: none
 * Returns:	nothing
 */


static int initialized=0;

void printfStringInit(int debug){
	UARTInit(0, 9600);	/* baud rate setting */
	if (debug ==1) UARTInit(3, 9600);	/* baud rate setting */
	initialized=1;
}

/*
 * void printfString1( int uart_number, const char * pcString)
 * Procedure that sends a \r\n terminated string message to the uart interface specified by uart_number, and optionally retrieves the response
 * Input parameters:
 * 		- uart_number: the number of the uart interface to send the string to
 * 		- pcString: string message that will be sent to the uart interface
 * Output parameters: none
 * Returns:	nothing
 */


void printfString1( int uart_number, const char * pcString)
{
	printfString2( uart_number, pcString, 1 );
}


/*
 * void printfStringAndNumber2( int uart_number, const char * pcString, unsigned long ulValue )
 * Procedure that sends an string message and a number to the uart interface specified by uart_number
 * Input parameters:
 * 		- uart_number: the number of the uart interface to send the string to
 * 		- pcString: string message that will be sent to the uart interface
 * 		- ulValue: number that will be printed to the uart interface next to the string message
 * Output parameters: none
 * Returns:	nothing
 */

void printfStringAndNumber2( int uart_number, const char * pcString, unsigned long ulValue )
{
	printfStringAndNumber3( uart_number, pcString, ulValue,1);
}

/*
 * void printfString2( int uart_number, const char * pcString, int terminate_string )
 * Procedure that sends an string message to the uart interface specified by uart_number, and optionally retrieves the response
 * Input parameters:
 * 		- uart_number: the number of the uart interface to send the string to
 * 		- pcString: string message that will be sent to the uart interface
 * 		- terminate_string: 1 if \r\n has to be added to terminate the string; 0 otherwise
 * Output parameters: none
 * Returns:	nothing
 */


void printfString2( int uart_number, const char * pcString, int terminate_string )
{
//	char ** resposta = 0;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;

	printfString( uart_number, pcString, terminate_string,0,dummy,0 );
	free(dummy);
}

/*
 * void printfStringAndNumber3(int uart_number, const char * pcString, unsigned long ulValue , int terminate_string)
 * Procedure that sends an string message and a number to the uart interface specified by uart_number
 * Input parameters:
 * 		- uart_number: the number of the uart interface to send the string to
 * 		- pcString: string message that will be sent to the uart interface
 * 		- ulValue: number that will be printed to the uart interface next to the string message
 * 		- terminate_string: 1 if \r\n has to be added to terminate the string; 0 otherwise
 * Output parameters: none
 * Returns:	nothing
 */
void printfStringAndNumber3(int uart_number, const char * pcString, unsigned long ulValue , int terminate_string)
{
	//char ** resposta = 0;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;

	printfStringAndNumber( uart_number, pcString, ulValue , terminate_string,0,dummy,0);
	free(dummy);
}


/*
 * void printfString( int uart_number, const char * pcString, int terminate_string, int retrieve_answer, char ** response, int delay_ms )
 * Procedure that sends an string message to the uart interface specified by uart_number, and optionally retrieves the response
 * Input parameters:
 * 		- uart_number: the number of the uart interface to send the string to
 * 		- pcString: string message that will be sent to the uart interface
 * 		- terminate_string: 1 if \r\n has to be added to terminate the string; 0 otherwise
 * 		- retrieve_answer: 1 if a response has to be retrieved from the uart interface; 0 otherwise
 * 		- delay_ms: time in ms that the routine will wait (passively) for the response to be received at the uart interface
 * 			before trying to retrieve it
 * Output parameters:
 * 		- response: pointer to the character pointer containing the response
 * Returns:	nothing
 */


void printfString( int uart_number, const char * pcString, int terminate_string, int retrieve_answer, char ** response, int delay_ms )
{
	if ( initialized==0)printfStringInit(1);

	static xSemaphoreHandle xMutex;
	if( xMutex== NULL){
		xMutex = xSemaphoreCreateMutex();
	}
	if( xMutex!= NULL){
		char format[10];
		format[0]='\0';
		const char * format1="%s";
		  strcpy(format,format1);

		if (terminate_string > 0 && (strlen(pcString) < BUFSIZE -1)){
			strcat(format,"\r\n");
		}

		xSemaphoreTake( xMutex, portMAX_DELAY );
		{
				cBuffer[0]='\0';
				int numchar=sprintf( cBuffer, format, pcString );
				cBuffer[numchar]='\0';
				UARTSend(uart_number, (uint8_t *)cBuffer , numchar );
		}
		xSemaphoreGive( xMutex );

		if (retrieve_answer > 0){
			*response=printf_captura_resposta2(uart_number,delay_ms);
		}else{
			*response=NULL;
		}

	}

}


/*
 * void printfStringAndNumber( int uart_number, const char *pcString, unsigned long ulValue , int terminate_string, int retrieve_answer, char ** response, int delay_ms)
 * Procedure that sends an string message and a number to the uart interface specified by uart_number, and optionally retrieves the response
 * Input parameters:
 * 		- uart_number: the number of the uart interface to send the string to
 * 		- pcString: string message that will be sent to the uart interface
 * 		- ulValue: number that will be printed to the uart interface next to the string message
 * 		- terminate_string: 1 if \r\n has to be added to terminate the string; 0 otherwise
 * 		- retrieve_answer: 1 if a response has to be retrieved from the uart interface; 0 otherwise
 * 		- delay_ms: time in ms that the routine will wait (passively) for the response to be received at the uart interface
 * 			before trying to retrieve it
 * Output parameters:
 * 		- response: pointer to the character pointer containing the response
 * Returns:	nothing
 */



void printfStringAndNumber( int uart_number, const char *pcString, unsigned long ulValue , int terminate_string, int retrieve_answer, char ** response, int delay_ms)
{
	char format[10];
	  format[0]='\0';
	  strcpy(format,"%s %lu");

	if (terminate_string > 0 && (strlen(pcString) < BUFSIZE -1)){
		strcat(format,"\r\n");
	}
	char buf[BUFSIZE];
	buf[0]='\0';
	int numchar=sprintf( buf, format, pcString, ulValue );
	if (numchar > 0)printfString( uart_number,  buf, terminate_string, retrieve_answer, response, delay_ms );

}


/*
 * void delayMs(portTickType ms)
 * Procedure implementing an active wait based on a simple loop
 * Input parameters:
 * 		- ms: time in ms that the routine will wait actively before continuing
 * Output parameters: none
 * Returns:	nothing
 */


void delayMs(portTickType ms){
	int i=0;
	for(i=0; i<(((uint32_t)7140)*(uint32_t)ms);i++){
		;
	}
}

/*
 * void delayMsX(int ms)
 * Procedure implementing an active wait based on a simple loop
 * Input parameters:
 * 		- ms: time in ms that the routine will wait actively before continuing
 * Output parameters: none
 * Returns:	nothing
 */


void delayMsX(int ms){
	uint32_t i=0;
	uint32_t j=0;
	for(i=0; i<(((uint32_t)7140)*(uint32_t)ms);i++){
		j++;
	}
}

/*
 * void printf_captura_resposta(int uart_number, char ** response, int delay_ms)
 * Procedure that scans for messages an uart interface specified by uart_number,
 * after waiting actively the time specified by the delay_ms parameter
 * Input parameters:
 * 		- uart_number: the number of the uart interface to retrieve the message from
 * 		- delay_ms: time in ms that the routine will wait (actively) for the message to be received at the uart interface
 * 			before trying to retrieve it
 * Output parameters:
 * 		- response: pointer to the character pointer containing the retrieved message
 * Returns:	nothing
 */


void printf_captura_resposta(int uart_number, char ** response, int delay_ms){
	if ( initialized==0)printfStringInit(1);

	portTickType wait_time= (portTickType) (delay_ms / portTICK_RATE_MS);
	static char buf[BUFSIZE];
	buf[0]='\0';
	int oldCount,newCount;
	oldCount=-1;
	newCount=0;
	delayMs(wait_time);//active wait

	  if ( uart_number == 0 )
	  {
			if ( UART0Count != 0 )
			{

				while(oldCount != newCount){
					  oldCount=newCount;
					  delayMs((portTickType)50);//active wait
					  newCount=UART0Count;
				  }

			  LPC_UART0->IER = IER_THRE | IER_RLS;			/* Disable RBR */
			  buf[0]='\0';
			  strncpy(buf,(char *)UART0Buffer,newCount);
			  buf[newCount]=0;
			  *response=(char *)buf;

			  UART0Count = 0;
			  LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */
			}

	  }
	  else if ( uart_number == 1 )
	  {
			if ( UART1Count != 0 )
			{

				while(oldCount != newCount){
					  oldCount=newCount;
					  delayMs((portTickType)50);//active wait
					  newCount=UART1Count;
				  }
			  LPC_UART1->IER = IER_THRE | IER_RLS;			/* Disable RBR */
			  buf[0]='\0';
			  strncpy(buf,(char *)UART1Buffer,newCount);
			  buf[newCount]='\0';
			  *response=(char *)buf;

			  UART1Count = 0;
			  LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */
			}

	  }
	  else if ( uart_number == 3 )
	  {
			if ( UART3Count != 0 )
			{
				while(oldCount != newCount){
					  oldCount=newCount;
					  delayMs((portTickType)50);//active wait
					  newCount=UART3Count;
				  }
			  LPC_UART3->IER = IER_THRE | IER_RLS;			/* Disable RBR */
			  buf[0]='\0';
			  strncpy(buf,(char *)UART3Buffer,newCount);
			  buf[newCount]='\0';
			  *response=(char *)buf;

			  UART3Count = 0;
			  LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */
			}
	  }
}

/*
 * char* printf_captura_resposta2(int uart_number, int delay_ms)
 * Procedure that scans for messages an uart interface specified by uart_number,
 * after waiting passively the time specified by the delay_ms parameter
 * Input parameters:
 * 		- uart_number: the number of the uart interface to retrieve the message from
 * 		- delay_ms: time in ms that the routine will wait (passively) for the message to be received at the uart interface
 * 			before trying to retrieve it
 * Output parameters: none
 * Returns:	character pointer containing the retrieved message
 */


char* printf_captura_resposta2(int uart_number, int delay_ms){
	if ( initialized==0)printfStringInit(1);

	portTickType wait_time= (portTickType) (delay_ms / portTICK_RATE_MS);
	static char buf[BUFSIZE];
	buf[0]='\0';
	int oldCount,newCount;
	oldCount=-1;
	newCount=0;

	portBASE_TYPE schedulerState = xTaskGetSchedulerState();
	if (schedulerState==taskSCHEDULER_RUNNING){//passive delay
		vTaskDelay(wait_time);
	}else{//active delay
		delayMs(wait_time);
	}


	  if ( uart_number == 0 )
	  {
			if ( UART0Count != 0 )
			{

				while(oldCount != newCount){
					  oldCount=newCount;
					  newCount=UART0Count;
					  //vTaskDelay((portTickType)50);//espera passiva
						if (schedulerState==taskSCHEDULER_RUNNING){//passive delay
							vTaskDelay((portTickType)50);
						}else{//active delay
							delayMs((portTickType)50);
						}

				}

			  LPC_UART0->IER = IER_THRE | IER_RLS;			/* Disable RBR */
			  buf[0]='\0';
			  strncpy(buf,(char *)UART0Buffer,newCount);
			  buf[newCount]=0;

			  UART0Count = 0;
			  LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */

			}

	  }
	  else if ( uart_number == 1 )
	  {
			if ( UART1Count != 0 )
			{

				while(oldCount != newCount){
					  oldCount=newCount;
					  //vTaskDelay((portTickType)50);//espera passiva
						if (schedulerState==taskSCHEDULER_RUNNING){//passive delay
							vTaskDelay((portTickType)50);
						}else{//active delay
							delayMs((portTickType)50);
						}
					  newCount=UART1Count;
				  }
			  LPC_UART1->IER = IER_THRE | IER_RLS;			/* Disable RBR */
			  buf[0]='\0';
			  strncpy(buf,(char *)UART1Buffer,newCount);
			  buf[newCount]='\0';

			  UART1Count = 0;

			  LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */
			}

	  }
	  else if ( uart_number == 3 )
	  {
			if ( UART3Count != 0 )
			{
				while(oldCount != newCount){
					  oldCount=newCount;
					  //vTaskDelay((portTickType)50);//espera passiva
						if (schedulerState==taskSCHEDULER_RUNNING){//passive delay
							vTaskDelay((portTickType)50);
						}else{//active delay
							delayMs((portTickType)50);
						}
					  newCount=UART3Count;
				  }
			  LPC_UART3->IER = IER_THRE | IER_RLS;			/* Disable RBR */
			  buf[0]='\0';
			  strncpy(buf,(char *)UART3Buffer,newCount);
			  buf[newCount]='\0';

			  UART3Count = 0;
			  LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */
			}


	  }
	  return &buf[0];
}

/*
 * char* printf_captura_UART0(int delay_ms)
 * Procedure that waits passively for messages at the UART0 interface
 * Input parameters:
 * 		- delay_ms: time in ms that the routine will wait (passively) between checks to the reception buffer
 * Output parameters: none
 * Returns:	character pointer containing the retrieved message
 */

char* printf_captura_UART0(int delay_ms){
	if ( initialized==0)printfStringInit(1);
	portTickType wait_time= (portTickType) (delay_ms / portTICK_RATE_MS);
	static char buf[BUFSIZE];
	buf[0]='\0';
	int oldCount,newCount;
	oldCount=-1;
	newCount=0;

	while( UART0Count == 0 ){
		vTaskDelay(wait_time);
		oldCount=newCount;
		newCount=UART0Count;
	}

	LPC_UART0->IER = IER_THRE | IER_RLS;			/* Disable RBR */
	buf[0]='\0';
	strncpy(buf,(char *)UART0Buffer,newCount);
	buf[newCount]=0;

	UART0Count = 0;
	LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */


	return &buf[0];
}

/*
 * char* chunk_line(const char * line)
 * Procedure that prepares a text string to be sent according to the chunked transfer protocol
 * The chucked line consists of the lenght of the string in hexadecimal format
 * followed by the string itself.
 * Input parameters:
 * 		- line: the char pointer with the string to format
 * Output parameters: none
 * Returns:	character pointer containing the reformatted string
 */

char* chunk_line(const char * line){
	int quant=0;
	static char buf[BUFSIZE];
	buf[0]='\0';

	quant=sprintf(buf,"%x", strlen(line));
	strcat(buf, "\r\n");
	strcat(buf, line);
	buf[quant+strlen(line)+2]='\0';
	 return &buf[0];
}
