/*
 * wifly.c
 *
 *  Created on: 04/11/2012
 *      Author: Cesar
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "FreeRTOS.h"
#include "task.h"
#include "uart.h"
#include "wifly.h"
#include "printf_library.h"

const char * connectstring="$$$";
const char * cmd="CMD";
const char * exit_command="exit";
const char * exit_ack="EXIT";
const char * ok="AOK";


extern volatile uint32_t UART3Count;
extern volatile uint8_t UART3Buffer[BUFSIZE];


/*
 * int wifly_reset()
 * Function that resets the Wifly module by sending a positive signal to its pin #5,
 * which is wired to pin #24 of the LPCXpresso board, for 2msec
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */

int wifly_reset()
{
	// Initialize pin
	LPC_PINCON->PINSEL1 &= ( ~( 3 << 12 ) );
	LPC_GPIO0->FIODIR |= ( 1 << RST_WIFLY_PIN );
	// it made reset at Wifly
	LPC_GPIO0->FIOCLR = 1 << RST_WIFLY_PIN;
	//vTaskDelay((portTickType)2); // min 160us
	portBASE_TYPE schedulerState = xTaskGetSchedulerState();
	if (schedulerState==taskSCHEDULER_RUNNING){//passive delay
		vTaskDelay((portTickType)10); // min 160us
	}else{//active delay
		delayMs((portTickType)10);
	}
	LPC_GPIO0->FIOSET = 1 << RST_WIFLY_PIN;

	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,4000);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta)>0){
		  return 0;
	  }

	return -1;
}

/*
 * int wifly_checkConnection ()
 * Function that checks the connection status of the Wifly module,
 * Using that the module responds with a <versionNumber> header
 * to an empty command line (\r\n)
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */

int wifly_checkConnection ()
{
	int retorn=-1;

	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,4000);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);

	  char less= '<';
	  char more= '>';



	  if(strlen(*resposta) >0 ){

		  char *  result1=(char *)strchr(*resposta,less);
		  char *  result2=(char *)strchr(*resposta,more);
		  if (result1 == NULL || result2 == NULL ){
			  retorn=-1;//not connected
		  }else{
			  retorn=0;
		  }
		  if (result1 != NULL)free(result1);
		  if (result2 != NULL)free(result2);
	  }else{
		  retorn=-1;
	  }
	  free(resposta);
	  free(dummy);
	  return retorn;//connected

}

/*
 * int wifly_connect ()
 * Function that sends the connect signal $$$ to the wifly module,
 * and waits for a "CMD" response acknowledging that command mode is activated
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */

int wifly_connect ()
{
	int retorn=-1;


	if (wifly_checkConnection()==0){
		return 0; //already connected
	}

	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , connectstring,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, connectstring,TERMINATE_STRING_NO,RETRIEVE_ANSWER_YES,resposta,1000);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);

	  if(strlen(*resposta) >0 ){

		 char * cerca= strstr(*resposta, cmd);
			if (cerca == NULL){
				retorn =-1;
			}else{
				free(cerca);
				retorn =0;
			}

	  }
	  free(resposta);
	  free(dummy);
	  return retorn;
}
/*
 * int wifly_disconnect ()
 * Function that sends the "exit" signal to the wifly module,
 * and waits for a "EXIT" response acknowledging that command mode is deactivated
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */

int wifly_disconnect ()
{
	int retorn=-1;
	if (wifly_checkConnection()==-1){
		return 0; //already disconnected
	}


	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	if (DEBUG_MODE==1)printfString(UART_DEBUG , exit_command,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	printfString(UART_WIFLY, exit_command,TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	if (strlen(*resposta)==0 ){
		retorn=0;//was not connected
	}else{

	}
	char * cerca = strstr(*resposta, "EXIT");
	if (cerca == NULL){
		retorn =-1;
	}else{
		free(cerca);
		retorn =0;
	}
	free(dummy);
	free(resposta);
	return retorn;
}

/*
 * int wifly_join (int order,const char * ssid, int security ,const char * pass)
 * Function that scans for available wifi networks and then connects
 * to the one specified by the entry parameters
 * Input parameters:
 * 		- order: position of the chosen network among those detected by the wifly module;
 * 		- ssid: identifier of the chosen network. It takes precedence over the previous parameter.
 * 		- security: 0(OPEN), 1(WEP), 2(WPA)
 * 		- pass:  wlan key, used if security is WEP or WPA
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */


int wifly_join (int order,const char * ssid, int security ,const char * pass)
{

	int result=0;
	if (wifly_checkConnection()==-1){
		int result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}
	char ** resposta= (char**)malloc(sizeof(char*));
	char * cerca= NULL;
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;

	if (order>0){//nou 29-12-12, no fer scan si ja sabem el nom de la xarxa
		if (DEBUG_MODE==1)printfString(UART_DEBUG , "scan",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
		printfString(UART_WIFLY, "scan",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,12000);
		if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);

		if(strlen(*resposta)==0){
			  free(resposta);
			  free(dummy);
			  return -1;
		}
	}
	char buffer[BUFSIZE];
	  buffer[0]='\0';

	if (security>OPEN)
		{
			strcpy(buffer,"set wlan key ");
			strcat(buffer,pass);
			*resposta=NULL;
			if (DEBUG_MODE==1)printfString(UART_DEBUG , buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
			printfString(UART_WIFLY, buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
			if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
			cerca = NULL;
			cerca = strstr(*resposta, ok);
			  if (cerca == NULL) {
				  free(buffer);
				  free(resposta);
				  free(dummy);
				  return -1;
			  }
		}

	  buffer[0]='\0';
	  strcpy(buffer,"join ");


	  if (strlen(ssid)>0)
		{
			strcat(buffer,ssid);
		}

	  else if (order>0)
		{
		  strcat(buffer,"# ");

		  char buffer2[4];
			buffer2[0]='\0';
			result=sprintf( buffer2, "%d", order );

		  strcat(buffer,buffer2);
		  free(buffer2);
		}
	  else {
		  free(buffer);
		  free(cerca);
		  free(resposta);
		  free(dummy);
		  return -1;
	  }
		*resposta=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,10000);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);

	  if (strlen(*resposta) > 0){
	  	  cerca = NULL;
		  cerca = strstr(*resposta, "IP=");
		  if (cerca == NULL) {
			  cerca = strstr(*resposta, "NM=");
		  }
		  if (cerca == NULL) {
			  cerca = strstr(*resposta, "GW=");
		  }
		  if (cerca == NULL) {
			  free(buffer);
			  free(resposta);
			  free(dummy);

			  return -1;
		  }else{
			  free(buffer);
			  free(cerca);
			  free(resposta);
			  free(dummy);
			  return 0;
		  }
	  }
		  return 0;
}

/*
 * int wifly_leave ()
 * Function that sends the "leave" signal to the wifly module
 * to abandon the current network connection
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */

int wifly_leave ()
{
	int retorn=-1;
	if (wifly_checkConnection()==-1){
		return 0; //already disconnected
	}


	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	if (DEBUG_MODE==1)printfString(UART_DEBUG , "leave",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	printfString(UART_WIFLY, "leave",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	if (strlen(*resposta)==0 ){
		retorn=0;//no estava connectat
	}else{

	}
	char * cerca = strstr(*resposta, "DeAuth");
	if (cerca == NULL){
		retorn =-1;
	}else{
		free(cerca);
		retorn =0;
	}
	free(dummy);
	free(resposta);
	return retorn;
}


/*
 * int wifly_ping (char * address)
 * Function that executes a ping command to a specified IP address or host name
 * Input parameters:
 * 		- address: remote ip address or dns name to send ping to;
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */


int wifly_ping (char * address){

		int result=wifly_connect();
		if (result<0) {
			return -1;
		}

	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;



	char * cerca = NULL;
	char buffer[BUFSIZE];
	buffer[0]='\0';
	  strcpy(buffer,"ping ");
	  strcat(buffer,address);
		char ** dummy=(char**)malloc(sizeof(char*));
		*dummy=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta)>0){
		  cerca = strstr(*resposta, ".");
		  if (cerca==NULL) {
			  free(buffer);
			  free(resposta);
			  free(dummy);
			  return -1;
		  }else{
			  free(cerca);
			  free(buffer);
			  free(resposta);
			  free(dummy);
			  return 0;
		  }
	  }else{
		  free(buffer);
		  free(resposta);
		  free(dummy);
		  return -1;
	  }
}

/*
 * int wifly_getIP (char * value)
 * Function that retrieves the IP adress from the wifly module
 * Input parameters: none
 * Output parameters:
 * 		- value: ip address as returned by the wifly module;
 * Returns:	0 if all goes well; -1 in case of error
 */


int wifly_getIP (char * value){

	int retorn=-1;

	value[0]='\0';
	if (wifly_checkConnection()==-1){
		int result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}

	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	if (DEBUG_MODE==1)printfString(UART_DEBUG , "get ip address ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	printfString(UART_WIFLY, "get ip address ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	if(strlen(*resposta)>0){
		  char * pch;
		  pch = strtok (*resposta,"\r\n");
		  while (strstr(pch, "get ip address") != NULL ){
			  pch = strtok (NULL, "\r\n");//l'ip estÃ  en la segona linia de la resposta, desprÃ©s de l'echo de comanda
		  }
		  if (pch !=NULL){
				value[0]='\0';
				strcpy(value,pch);
				retorn=strlen(pch);
				free(pch);
				free(resposta);
				free(dummy);
				return retorn;
		  }else{
				free(resposta);
				free(dummy);
				return -1;
		  }

	}else{
		free(resposta);
		free(dummy);
		return -1;
	}


}

/*
 * int wifly_getChannel (char * value)
 * Function that checks the wlan configuration of the wifly module
 * and retrieves the channel currently used
 * Input parameters: none
 * Output parameters:
 * 		- value: channel currently used by the active wlan connection of the wifly module;
 * Returns:	0 if all goes well; -1 in case of error
 */


int wifly_getChannel (char * value){
	value[0]='\0';
	int retorn = -1;
	if (wifly_checkConnection()==-1){
		int result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	if (DEBUG_MODE==1)printfString(UART_DEBUG , "get wlan",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	printfString(UART_WIFLY, "get wlan ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,0);
	if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	if(strlen(*resposta)>0){
		  char * pch;
		  pch = strtok (*resposta,"\r\n");
		  while (pch !=NULL){
		  	  if (strstr(pch, "Chan=") != NULL ){//looks for the line where the channel is listed
		  		  pch = strtok (pch, "=");
		  		  if(pch !=NULL)pch = strtok (NULL, "=");
		  		  break;
		  	  }
		  	  pch = strtok (NULL, "\r\n");
		  }
		  if (pch !=NULL){
				value[0]='\0';
				strcpy(value,pch);
				retorn=strlen(pch);
				free(pch);
				free(resposta);
				free(dummy);
				return retorn;
		  }else{
				free(resposta);
				free(dummy);
				return -1;
		  }
	}else{
		free(resposta);
		free(dummy);
		return -1;
	}
}

/*
 * int wifly_getTXPower (char * value)
 * Function that checks the wlan configuration of the wifly module
 * and retrieves the current transmission power level (0-12)
 * Input parameters: none
 * Output parameters:
 * 		- value: current wlan transmission power level of the wifly module (0-12);
 * Returns:	0 if all goes well; -1 in case of error
 */

int wifly_getTXPower (char * value){
	value[0]='\0';
	int retorn = -1;

	if (wifly_checkConnection()==-1){
		int result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	if (DEBUG_MODE==1)printfString(UART_DEBUG , "get wlan",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	printfString(UART_WIFLY, "get wlan ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,500);
	if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	if(strlen(*resposta)>0){
		  char * pch;
		  pch = strtok (*resposta,"\r\n");
		  while (pch !=NULL){
		  	  if (strstr(pch, "wer=") != NULL ){//looks for the line where the tx power is listed
		  		  pch = strtok (pch, "=");
		  		  if(pch !=NULL)pch = strtok (NULL, "=");
		  		  break;
		  	  }
		  	  pch = strtok (NULL, "\r\n");
		  }
		  if (pch !=NULL){
				value[0]='\0';
				strcpy(value,pch);
				retorn=strlen(pch);
				free(pch);
				free(resposta);
				free(dummy);
				return retorn;
		  }else{
				free(resposta);
				free(dummy);
				return -1;
		  }
	}else{
		free(resposta);
		free(dummy);
		return -1;
	}
}

/*
 * int wifly_getRSSI (char * value)
 * Function that retrieves the RSSI parameter of the wifly module
 * Input parameters: none
 * Output parameters:
 * 		- value: current RSSI value in db;
 * Returns:	0 if all goes well; -1 in case of error
 */


int wifly_getRSSI (char * value){
	int retorn =-1;
	value[0]='\0';
	if (wifly_checkConnection()==-1){
		int result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	if (DEBUG_MODE==1)printfString(UART_DEBUG , "show rssi",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	printfString(UART_WIFLY, "show rssi",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,200);//1200
	if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	if(strlen(*resposta)>0){
		  char * pch;
		  pch = strtok (*resposta,"\r\n");
		  while (pch !=NULL){
		  	  if (strstr(pch, "RSSI=") != NULL ){//extracts the value from the line, which has format "RSSI(value) db"
		  		  pch = strtok (pch, "=()");
		  		  if(pch !=NULL)pch = strtok (NULL, "=()");
		  		  break;
		  	  }
		  	  pch = strtok (NULL, "\r\n");
		  }
		  if (pch !=NULL){
				value[0]='\0';
				strcpy(value,pch);
				retorn=strlen(pch);
				free(pch);
				free(resposta);
				free(dummy);
				return retorn;
		  }else{
				free(resposta);
				free(dummy);
				return -1;
		  }
	}else{
		free(resposta);
		free(dummy);
		return -1;
	}
}

/*
 * int wifly_getRSSIint ()
 * Function that retrieves the RSSI parameter of the wifly module
 * Input parameters: none
 * Output parameters: none
 * Returns:	-1000 in case of error; current RSSI value in db otherwise;
 */


int wifly_getRSSIint(int slow) {
	char value[10];
	int result=-1;
	int valor=0;
	char buffer2[20];

	if (slow==1){
		//since the rssi measurement has low repeatability, it is hard to use it to look for the best coverage
		//we try to improve this by averaging a series of measurements.
		//It's not adequate to repeate calls to wifly_getRSS as the overhead of the connect() call will make it extremely slow
		result=wifly_connect();
		if (result<0) {
			return -1;
		}
		int j=0;
		int rssi[5];

		char ** resposta= (char**)malloc(sizeof(char*));
		*resposta=NULL;
		char * pch;

		while (j<5){
			printfString(UART_WIFLY, "show rssi",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,200);//1200
			if(strlen(*resposta)>0){
				  pch = strtok (*resposta,"\r\n");
				  while (pch !=NULL){
					  if (strstr(pch, "RSSI=") != NULL ){//extracts the value from the line, which has format "RSSI(value) db"
						  pch = strtok (pch, "=()");
						  if(pch !=NULL)pch = strtok (NULL, "=()");
						  break;
					  }
					  pch = strtok (NULL, "\r\n");
				  }
				  if (pch !=NULL){
						value[0]='\0';
						strcpy(value,pch);
						//if (DEBUG_MODE==1)printfString1(UART_DEBUG , value);
						rssi[j] = (int)strtol(value,NULL,10);
						j++;
				  }
			}
		}
		free(pch);
		free(resposta);
		//now we have 5 rssi readings. We discard the highest and lowest and average the rest
		int mx=-1000;
		int mn=1000;
		int mxj=0;
		int mnj=0;
		int sum=0;
		int ave=0;
		for(j=0; j <5;j++){
			if (rssi[j]>mx){
				mx=rssi[j];
				mxj=j;
			}
			if (rssi[j]<mn){
				mn=rssi[j];
				mnj=j;
			}
		}
		for(j=0; j <5;j++){
			if (j != mxj && j != mnj){
				sum=sum+rssi[j];
			}
		}

		if (mxj == mnj){
			ave=sum / 4;
		}else{
			ave=sum / 3;
		}

		valor =ave;

		buffer2[0]='\0';
		result=sprintf( buffer2, "RSSI: %d dBm", valor );
		if (DEBUG_MODE==1)printfString1(UART_DEBUG , buffer2);
	}else{
		char buffer[10];
		buffer[0]='\0';

		result=wifly_getRSSI(buffer);
		if (result >= 0 ) {
			valor = (int)strtol(buffer,NULL,10);
		}else{
			//retry once
			result=wifly_getRSSI(buffer);
			if (result >= 0 ) {
				valor = (int)strtol(buffer,NULL,10);
			}else{
				valor=-1000;
			}
		}

	}
		return valor;
}


/*
 * int wifly_getTime (char * value)
 * Function that retrieves the wifly module up time in seconds
 * Input parameters: none
 * Output parameters:
 * 		- value: time since the wifly module was powered up, in seconds;
 * Returns:	0 if all goes well; -1 in case of error
 */


int wifly_getTime (char * value){
	value[0]='\0';
	int retorn = -1;

	if (wifly_checkConnection()==-1){
		int result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	if (DEBUG_MODE==1)printfString(UART_DEBUG , "show time",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	printfString(UART_WIFLY, "show time",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	if(strlen(*resposta)>0){
		  char * pch;
		  pch = strtok (*resposta,"\r\n");
		  while (pch !=NULL){
		  	  if (strstr(pch, "UpTime=") != NULL ){
		  		  pch = strtok (pch, "= ");
		  		  if(pch !=NULL)pch = strtok (NULL, "= ");
		  		  break;
		  	  }
		  	  pch = strtok (NULL, "\r\n");
		  }
		  if (pch !=NULL){
				value[0]='\0';
				strcpy(value,pch);
				retorn=strlen(pch);
				free(pch);
				free(resposta);
				free(dummy);
				return retorn;
		  }else{
				free(resposta);
				free(dummy);
				return -1;
		  }
	}else{
		free(resposta);
		free(dummy);
		return -1;
	}
}

/*
 * int wifly_setTxPower (int power)
 * Function that sets the wlan transmission power level (0-12) of the wifly module
 * Input parameters:
 * 		- power: transmission power level (0-12) of the wifly module;
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */

int wifly_setTXPower(int power){
	int result=-1;
	if (wifly_checkConnection()==-1){
		result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}

	if (power < 0 || power>12){//rebutgem valors incorrectes
		return -1;
	}


		char buffer[BUFSIZE];
		buffer[0]='\0';
		char buffer2[4];
		buffer2[0]='\0';
		result=sprintf( buffer2, "%d", power );

		strcpy(buffer,"set wlan tx ");
		  strcat(buffer,buffer2);
			char ** resposta= (char**)malloc(sizeof(char*));
			*resposta=NULL;
			char ** dummy=(char**)malloc(sizeof(char*));
			*dummy=NULL;
		  if (DEBUG_MODE==1)printfString(UART_DEBUG , "",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
		  printfString(UART_WIFLY, buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
		  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);


		  if (strlen(*resposta) > 0){
			  char * cerca = strstr(*resposta, ok);
			  if (cerca == NULL) {
				  free(buffer);
				  free(buffer2);
				  free(resposta);
				  free(dummy);

				  return -1;
			  }else{
				  free(buffer);
				  free(buffer);
				  free(cerca);
				  free(resposta);
				  free(dummy);
				  return 0;
			  }
		  }

		  free(buffer);
		  free(buffer);
		  free(resposta);
		  free(dummy);
		  return -1;

}


/*
 * int wifly_setHttpConfig ()
 * Function that sets the following http configuration parameters of the wifly module:
 *		set ip proto 18 -> protocol = TCP+HTTP
 *		set ip host 0 -> host is specified by dns name
 *		set ip remote 80 -> connect to host on standard http port 80
 *		set comm size 128 -> When 128 bytes of data are available, send an IP packet. If this parameter
 *			is left at its default value, sending of long GET requests will crash.
 *		set com remote 0 -> GET requests will not be stored in the com remote parameter, but pasted directly
 *			in the tcp session.
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */


int wifly_setHttpConfig (){
	int result=0;
	if (wifly_checkConnection()==-1){
		result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;

	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;

	char * cerca= NULL;


	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "set ip proto 18",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "set ip proto 18 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		 cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }

	  *resposta=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "set ip host 0",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "set ip host 0 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }
	  *resposta=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "set ip remote 80",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "set ip remote 80 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  //free(cerca);
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }

	  *resposta=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "set comm size 128",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "set comm size 128 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }
	  *resposta=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "set com remote 0",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "set com remote 0",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,500);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }

	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }

	  //allibera recursos
	  *resposta=NULL;
	  if (cerca != NULL) free(cerca);
	  free(resposta);
	  free(dummy);
	  return 0;
}

/*
 * int wifly_sendHttpGET (const char * host,const char * getCommand, char * retorn)
 * Function that opens a tcp connection to the specified host, sends a GET request,
 * retrieves the response, and closes the connection
 * Input parameters:
 * 		- host: ip or host name to connect to;
 * 		- getCommand: string with the complete GET request with parameters and headers
 * Output parameters:
 * 		- retorn: text returned by the host
 * Returns:	number of characters in the response to the request, if all goes well; -1 in case of error
 */


int wifly_sendHttpGET (const char * host,const char * getCommand, char * retorn){
	retorn[0]='\0';
	int result=0;

	if (wifly_checkConnection()==-1){
		result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}

	if (strlen(getCommand)>BUFSIZE){//rebutgem requests massa llargs
		return -1;
	}

	char * cerca= NULL;
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char buffer[BUFSIZE];
	buffer[0]='\0';

	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;

	  strcpy(buffer,"set dns name ");
	  strcat(buffer,host);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,500);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(buffer);
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(buffer);
		  free(resposta);
		  free(dummy);
		  return -1;
	  }
	  *resposta=NULL;
	  buffer[0]='\0';

	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "open",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);

	  printfString(UART_WIFLY, "open ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,2000);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);

	  cerca = strstr(*resposta, "ERR:Connected!");//connexio ja estava oberta
	  if (cerca != NULL) {
		  if (DEBUG_MODE==1)printfString(UART_DEBUG , "close",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
		  printfString(UART_WIFLY, "close ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,2000);
		  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
		  if (DEBUG_MODE==1)printfString(UART_DEBUG , "open",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
		  printfString(UART_WIFLY, "open ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,2000);
		  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  }
	  int comptador=0;
	  for (comptador=0;comptador<30; comptador++ ){
		  cerca = strstr(*resposta, "*OPEN*");//posem el GET quan ja s'ha obert l'OPEN

		  if (cerca == NULL) {
			  printfString(UART_WIFLY, "",TERMINATE_STRING_NO,RETRIEVE_ANSWER_YES,resposta,1000);//resposta *OPEN* no ha arribat encara; enviem cadena buida per seguir escanejant sortida
		  }else{
			  *resposta=NULL;

			  if (DEBUG_MODE==1)printfString(UART_DEBUG , getCommand,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
			  printfString(UART_WIFLY, getCommand,TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,5000);
			  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
			  if (strlen(*resposta) > 0){
				  retorn[0]='\0';
				  strcpy(retorn,*resposta);
				  int nchars=strlen(*resposta);
				  wifly_connect();
				  *resposta=NULL;
				  if (DEBUG_MODE==1)printfString(UART_DEBUG , "close",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
				  printfString(UART_WIFLY, "close ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,2000);
				  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
				  free(buffer);
				  free(resposta);
				  free(dummy);
				  return nchars;//retornem el nombre de carÃ cters a la resposta
			  }
			  break;
		  }


	  }
	  if (cerca != NULL) free(cerca);
	  free(buffer);
	  free(resposta);
	  free(dummy);
	  return -1;
}

/*
 * int wifly_sendHttpGET2 (const char * host,const char * getCommand, char * retorn)
 * alternate to Function sendHttpGET
 * The GET command is stored in the com remote parameter of the wifly module
 * which is limited to 64 characters. So, longers requests can not be processed
 * Input parameters:
 * 		- host: ip or host name to connect to;
 * 		- getCommand: string with the complete GET request with parameters and headers
 * Output parameters:
 * 		- retorn: text returned by the host
 * Returns:	number of characters in the response to the request, if all goes well; -1 in case of error
 */


int wifly_sendHttpGET2 (const char * host,const char * getCommand, char * retorn){
	retorn[0]='\0';
	int result=0;

	if (wifly_checkConnection()==-1){
		result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}

	if (strlen(getCommand)>64){//rebutgem requests massa llargs
		return -1;
	}

	char * cerca= NULL;
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char buffer[BUFSIZE];
	buffer[0]='\0';

	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;

	  strcpy(buffer,"set dns name ");
	  strcat(buffer,host);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,500);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(buffer);
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(buffer);
		  free(resposta);
		  free(dummy);
		  return -1;
	  }
	  *resposta=NULL;
	  buffer[0]='\0';

	  strcpy(buffer,"set com remote ");
	  strcat(buffer,getCommand);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, buffer,TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,2000);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(buffer);
			  free(resposta);
			  free(dummy);
			  return -1;
		  }

	  }else{
		  free(buffer);
		  free(resposta);
		  free(dummy);
		  return -1;
	  }
	  *resposta=NULL;
	  buffer[0]='\0';

	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "open",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);

	  printfString(UART_WIFLY, "open ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,3000);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){//recollim resposta
		  retorn[0]='\0';
		  strcpy(retorn,*resposta);
		  int nchars=strlen(*resposta);

		  wifly_connect();
		  *resposta=NULL;
		  //tanquem connexio
		  if (DEBUG_MODE==1)printfString(UART_DEBUG , "close",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
		  printfString(UART_WIFLY, "close ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1000);
		  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
		  free(buffer);
		  free(resposta);
		  free(dummy);
		  return nchars;
	  }


	  if (cerca != NULL) free(cerca);
	  free(buffer);
	  free(resposta);
	  free(dummy);
	  return -1;
}
/*
 * int wifly_setCommTime ()
 * Function that sets the following http configuration parameters of the wifly module:
 *		set comm time 2000 -> Auto send an IP packet if 2000 ms have elapsed since the tcp session was opened
 *			or the last packet was sent
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */

int wifly_setCommTime (){
	int result=0;
	if (wifly_checkConnection()==-1){
		result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;

	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;

	char * cerca= NULL;


	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "set comm time 2000 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "set comm time 2000 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		 cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }

	  *resposta=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "save ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "save ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }
	  *resposta=NULL;
	  if (cerca != NULL) free(cerca);
	  free(resposta);
	  free(dummy);
	  return 0;
}

/*
 * int wifly_getTime (char * value)
 * Function that retrieves the comm time http configuration parameter from the wifly module
 * Input parameters: none
 * Output parameters:
 * 		- value: waiting time when a tcp session is opened until an ip packet is sent
 * Returns:	0 if all goes well; -1 in case of error
 */

int wifly_getCommTime (char * value){
	value[0]='\0';
	int retorn = -1;

	if (wifly_checkConnection()==-1){
		int result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	if (DEBUG_MODE==1)printfString(UART_DEBUG , "get comm time",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	printfString(UART_WIFLY, "get comm time time",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	if(strlen(*resposta)>0){
		value[0]='\0';
		strcpy(value,*resposta);
		retorn=strlen(*resposta);
		free(resposta);
		free(dummy);
		return retorn;

	}else{
		free(resposta);
		free(dummy);
		return -1;
	}
}
/*
 * int wifly_setCommSize ()
 * Function that sets the following http configuration parameters of the wifly module:
 *		set comm size 128 -> When 128 bytes of data are available, send an IP packet. If this parameter
 *			is left at its default value, sending of long GET requests will crash.
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */
int wifly_setCommSize (){
	int result=0;
	if (wifly_checkConnection()==-1){
		result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;

	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;

	char * cerca= NULL;

	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "set comm size 128 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "set comm size 128 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		 cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }

	  *resposta=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "save ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "save ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }
	  *resposta=NULL;
	  if (cerca != NULL) free(cerca);
	  free(resposta);
	  free(dummy);
	  return 0;
}

/*
 * int wifly_getCommSize (char * value)
 * Function that retrieves the comm time http configuration parameter from the wifly module
 * Input parameters: none
 * Output parameters:
 * 		- value: number of bytes that are accumulated in the uffer until an IP packet is sent
 * Returns:	0 if all goes well; -1 in case of error
 */

int wifly_getCommSize (char * value){
	value[0]='\0';
	int retorn = -1;

	if (wifly_checkConnection()==-1){
		int result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;
	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;
	if (DEBUG_MODE==1)printfString(UART_DEBUG , "get comm size",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	printfString(UART_WIFLY, "get comm size ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	if(strlen(*resposta)>0){
		value[0]='\0';
		strcpy(value,*resposta);
		retorn=strlen(*resposta);
		free(resposta);
		free(dummy);
		return retorn;

	}else{
		free(resposta);
		free(dummy);
		return -1;
	}
}

/*
 * int wifly_setTcpServerConfig ()
 * Function that sets the following tcp configuration parameters of the wifly module:
 *		set ip proto 2 -> protocol = TCP server and client
 *		set ip local 80 -> listen to tcp connections on standard http port 80
 *		set comm size 128 -> When 128 bytes of data are available, send an IP packet. If this parameter
 *			is left at its default value, sending of long GET requests will crash.
 *		set comm remote 0 -> do not send a *HELLO* prompt to the client that opens a tcp connection
 * The configuration must be saved and the wifly module must be rebooted
 * Otherwise the set ip local 80 command will not take effect
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */


int wifly_setTcpServerConfig(){
	int result=0;
	if (wifly_checkConnection()==-1){
		result=wifly_connect();
		if (result<0) {
			return -1;
		}
	}
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;

	char ** dummy=(char**)malloc(sizeof(char*));
	*dummy=NULL;

	char * cerca= NULL;


	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "set ip proto 2",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "set ip proto 2 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		 cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }

	  *resposta=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "set ip local 80",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "set ip local 80 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }

	  *resposta=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "set comm size 128",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "set comm size 128 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }
	  //comm remote 0 perquè no respongui *HELLO* a la connexio TCP
	  *resposta=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "set comm remote 0 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "set comm remote 0 ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,1200);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, ok);
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }
	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }


	  *resposta=NULL;
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , "save",TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  printfString(UART_WIFLY, "save",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,500);
	  if (DEBUG_MODE==1)printfString(UART_DEBUG , *resposta,TERMINATE_STRING_YES,RETRIEVE_ANSWER_NO,dummy,0);
	  if (strlen(*resposta) > 0){
		  cerca = strstr(*resposta, "in config");
		  if (cerca == NULL) {
			  free(resposta);
			  free(dummy);
			  return -1;
		  }

	  }else{
		  free(resposta);
		  free(dummy);
		  return -1;
	  }

	  //allibera recursos
	  *resposta=NULL;
	  if (cerca != NULL) free(cerca);
	  free(resposta);
	  free(dummy);

	  int ret=wifly_reset();//reiniciem dispositiu perquè el canvi de port sigui efectiu
	  return ret;
}
/*
 * int wifly_closeTcpConnection()
 * Function that closes a tcp connection from the wifly module side
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */


int wifly_closeTcpConnection(){
	int result=0;
		result=wifly_connect();
		if (result<0) {
			return -1;
		}
	char ** resposta= (char**)malloc(sizeof(char*));
	*resposta=NULL;


	char * cerca= NULL;
	  //if (DEBUG_MODE==1)printfString1(UART_DEBUG , "close");
	  printfString(UART_WIFLY, "close ",TERMINATE_STRING_YES,RETRIEVE_ANSWER_YES,resposta,2000);
	  if (DEBUG_MODE==1)printfString1(UART_DEBUG , *resposta);

	  cerca = strstr(*resposta, "*CLOS*");

	  if (cerca == NULL) {
		  free(resposta);
		  return -1;
	  }else{
		  printfString1(UART_WIFLY, "");
		  free(cerca);
		  free(resposta);
		  return 0;
	  }

}
